@@ -4,7 +4,10 @@
 This module includes building blocks and storage implementations of **Ring**
 factories for :mod:`asyncio`.
 """
-from typing import Any, Optional, List
+import sys
+from functools import wraps, partial
+
+from ring.typing import Any, Optional, List
 import asyncio
 import inspect
 import itertools
@@ -29,16 +32,18 @@ class SingletonCoroutineProxy(object):
         self.awaitable = awaitable
         self.singleton = None
 
-    def __iter__(self):
-        with (yield from lock):
+    async def _await(self):
+        async with lock:
             if self.singleton is None:
                 if hasattr(self.awaitable, '__await__'):
-                    awaitable = self.awaitable.__await__()
+                    self.singleton = await self.awaitable
                 else:
-                    awaitable = self.awaitable
-                self.singleton = yield from awaitable
+                    self.singleton = self.awaitable
         return self.singleton
 
+    def __iter__(self):
+        return self._await().__await__()
+
     __await__ = __iter__
 
 
@@ -69,6 +74,16 @@ def create_asyncio_factory_proxy(factory_table, *, support_asyncio):
     return fbase.create_factory_proxy(proxy_base, classifier, factory_table)
 
 
+def async_wrap(func):
+    @wraps(func)
+    async def run(*args, _executor=None, **kwargs):
+        loop = asyncio.get_event_loop()
+        pfunc = partial(func, *args, **kwargs)
+        return await loop.run_in_executor(_executor, pfunc)
+
+    return run
+
+
 def convert_storage(storage_class):
     storage_bases = (fbase.CommonMixinStorage, BulkStorageMixin)
     async_storage_class = type(
@@ -79,7 +94,10 @@ def convert_storage(storage_class):
         if issubclass(storage_class, storage_base):
             count += 1
             for name in storage_base.__dict__.keys():
-                async_attr = asyncio.coroutine(getattr(storage_class, name))
+                if sys.version_info < (3, 8):
+                    async_attr = asyncio.coroutine(getattr(storage_class, name))
+                else:
+                    async_attr = async_wrap(getattr(storage_class, name))
                 setattr(async_storage_class, name, async_attr)
     if count == 0:
         raise TypeError(
@@ -146,7 +164,7 @@ class CacheUserInterface(fbase.BaseUserInterface):
     """
 
     @fbase.interface_attrs(
-        return_annotation=lambda a: Optional[a.get('return', Any)])
+        return_annotation=lambda a: Optional[a.get('return', Any)])  # noqa: F722
     async def get(self, wire, **kwargs):
         key = self.key(wire, **kwargs)
         try:
@@ -205,14 +223,14 @@ class BulkInterfaceMixin(fbase.AbstractBulkUserInterfaceMixin):
 
     @fbase.interface_attrs(
         transform_args=fbase.transform_positional_only,
-        return_annotation=lambda a: List[a.get('return', Any)])
+        return_annotation=lambda a: List[a.get('return', Any)])  # noqa: F722
     def execute_many(self, wire, pargs):
         return asyncio.gather(*(
             fbase.execute_bulk_item(wire, args) for args in pargs.args))
 
     @fbase.interface_attrs(
         transform_args=fbase.transform_positional_only,
-        return_annotation=lambda a: List[Optional[a.get('return', Any)]])
+        return_annotation=lambda a: List[Optional[a.get('return', Any)]])  # noqa: F722
     def get_many(self, wire, pargs):
         keys = self.key_many(wire, pargs)
         return wire.storage.get_many(
@@ -220,7 +238,7 @@ class BulkInterfaceMixin(fbase.AbstractBulkUserInterfaceMixin):
 
     @fbase.interface_attrs(
         transform_args=fbase.transform_positional_only,
-        return_annotation=lambda a: List[a.get('return', Any)])
+        return_annotation=lambda a: List[a.get('return', Any)])  # noqa: F722
     async def update_many(self, wire, pargs):
         keys = self.key_many(wire, pargs)
         values = await self.execute_many(wire, pargs)
@@ -229,7 +247,7 @@ class BulkInterfaceMixin(fbase.AbstractBulkUserInterfaceMixin):
 
     @fbase.interface_attrs(
         transform_args=fbase.transform_positional_only,
-        return_annotation=lambda a: List[a.get('return', Any)])
+        return_annotation=lambda a: List[a.get('return', Any)])  # noqa: F722
     async def get_or_update_many(self, wire, pargs):
         keys = self.key_many(wire, pargs)
         miss_value = object()
